/*
 * ===========================
 * VDK Visual Development Kit
 * odbc subsystem
 * Version 0.1
 * Revision 0.0
 * January 2001
 * ===========================
 *
 * Copyright (C) 2001, Mario Motta
 * Developed by Pierre-Louis Malatray <pierrelouis.malatray@free.fr>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
 * 02111-1307, USA.
 */
#ifndef _vdkodbc_h
#define _vdkodbc_h

#include <glib.h>
extern "C"
{
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <stdarg.h>		// for printf capability
}

#if defined _WIN32 || defined _WIN64
// SQL types for DEV-C++
typedef unsigned long ULONG;
typedef unsigned short USHORT;
typedef void* PVOID;
typedef unsigned char UCHAR;
typedef long DWORD;
typedef int WORD;
typedef char BYTE;
typedef bool BOOL;
typedef char* LPWSTR;
typedef void VOID;
typedef unsigned long HWND;
#endif // _WIN32
 
// unixODBC or DEV-C++
#include <sql.h>
#include <sqlext.h>
#include <sqltypes.h>

// vdk
#include <vdk/vdkstring.h>

//! Internal errors
#define VDKODBC_ERR_BASE				-1000
#define VDKODBC_ERR_MEMORY				VDKODBC_ERR_BASE
#define VDKODBC_ERR_INVALID_CONN_POINTER		VDKODBC_ERR_BASE-1
#define VDKODBC_ERR_INVALID_COLUMN_NAME			VDKODBC_ERR_BASE-2
#define VDKODBC_ERR_INVALID_COLUMN_NUMBER		VDKODBC_ERR_BASE-3
#define VDKODBC_ERR_NULL_FIELD_LENGTH			VDKODBC_ERR_BASE-4
#define VDKODBC_ERR_EOF_REACHED				VDKODBC_ERR_BASE-5
#define VDKODBC_ERR_CONN_NEEDED				VDKODBC_ERR_BASE-6
#define VDKODBC_ERR_TABLE_NOT_FOUND			VDKODBC_ERR_BASE-7
#define VDKODBC_ERR_COLUMN_NOT_FOUND			VDKODBC_ERR_BASE-8

//! Answers
#define VDKODBC_NULLABLE				1
#define VDKODBC_NOT_NULLABLE				0

//! Constants
#define VDKODBC_MAX_ERR_BUF				1025
#define VDKODBC_MAX_BUF					2049
#define VDKODBC_BUF					129
#define VDKODBC_MAXCOLNAMELEN				256		// 32 for PostGreSQL
#define VDKODBC_FETCH_ALL				0
#define VDKODBC_FETCH_USER				1
#define VDKODBC_FETCH_SYSTEM				2
//! Objects fields position
//! Privileges
#define VDKODBC_PRIV_GRANTOR				1
#define VDKODBC_PRIV_GRANTEE				2
#define VDKODBC_PRIV_PRIV				3
#define VDKODBC_PRIV_GRANTABLE				4
//! Base
#define VDKODBC_NAME					1
#define VDKODBC_TYPENAME				2
#define VDKODBC_REMARKS					3
//! Table
#define VDKODBC_TABLE_CAT				4
#define VDKODBC_TABLE_SCH				5
//! Column
#define VDKODBC_COL_TYPE				6
#define VDKODBC_COL_SIZE				7
#define VDKODBC_COL_BUFLEN				8
#define VDKODBC_COL_DECDIGITS				9
#define VDKODBC_COL_RADIX				10
#define VDKODBC_COL_NULLABLE				11
#define VDKODBC_COL_DEF					12
#define VDKODBC_COL_SQLTYPE				13
#define VDKODBC_COL_SQLDTSUB				14
#define VDKODBC_COL_BYTELEN				15
#define VDKODBC_COL_ORDPOS				16
#define VDKODBC_COL_ISNULLABLE				17

/*!
\class VDKODBCObject
\brief Ancestor for all VDKODBC objects hierarchy
*/
class VDKODBCObject
{
	protected:
		//! Last error number
		SQLINTEGER 	LastNumError;
		//! Last error message buffer
		VDKString LastError;
		/*!
		Tests for ODBC function result and
		provides last error number and message
		\param rslt Value returned by an ODBC function
		\param henv Environment ODBC handle
		\param hconn Connection ODBC handle
		\param hstmt Statement ODBC handle
		\return true if the operation was successfully terminated
		*/
		bool SQLTest(SQLINTEGER rslt,
					 SQLHENV henv = SQL_NULL_HENV,
					 SQLHDBC hconn = SQL_NULL_HDBC,
					 SQLHSTMT hstmt = SQL_NULL_HSTMT);
		/*!
		Translate internal error number in an error string and set LastNumError \n
		and LastError. Supports printf facility
		\param errnum Error number
		\param format String containing format flags
		\param ... Variable parameter list
		*/
		void Err(SQLINTEGER errnum, const char *format = NULL, ...);	
	public:
		/*!
		Constructor
		*/
		VDKODBCObject() {
			LastNumError = SQL_SUCCESS;
		};
		/*!
		Destructor
		*/
		~VDKODBCObject() {};
		/*!
		Returns the last error number
		*/
		int GetLastNumError() {
			return LastNumError;
		};
		/*!
		Returns the last error text
		*/
		VDKString GetLastError() {
			return LastError;
		};
		/*!
		Tests for ODBC function result
		\param rslt Value returned by an ODBC function
		\return true if the operation was successfully terminated
		*/
		bool SQL_OK(SQLINTEGER rslt);
};

#endif



